/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.event;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.test.*;

/**
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class FrameListener_Test extends APITestCase {

    public void testFrameNameChangedEvent() {
        clearEvents();
        FrameListener listener = new FrameAdapter() {
            public void nameChanged(FrameEvent event) {
                recordEventFired(event);
            }
            public void ownSlotValueChanged(FrameEvent event) {
                recordEventFired(event);
            }
            public void browserTextChanged(FrameEvent event) {
                recordEventFired(event);
            }
        };
        Frame frame = createCls();
        frame.addFrameListener(listener);
        frame.setName("foo");
        assertEventFired(FrameEvent.NAME_CHANGED);
        assertEventFired(FrameEvent.OWN_SLOT_VALUE_CHANGED);
        assertEventFired(FrameEvent.BROWSER_TEXT_CHANGED);
    }

    public void testOwnSlotValueChangedEvent() {
        clearEvents();
        FrameListener listener = new FrameAdapter() {
            public void ownSlotValueChanged(FrameEvent event) {
                recordEventFired(event);
            }
        };
        Cls cls = createCls();
        Slot slot = createSlot();
        cls.addDirectTemplateSlot(slot);
        Instance instance = createInstance(cls);
        instance.addFrameListener(listener);
        instance.setOwnSlotValue(slot, "foo");
        assertEventFired(FrameEvent.OWN_SLOT_VALUE_CHANGED);
        clearEvents();
        instance.setOwnSlotValues(slot, makeList("a", "b"));
        FrameEvent event = (FrameEvent) getEventFired(FrameEvent.OWN_SLOT_VALUE_CHANGED);
        assertEqualsList(event.getOldValues(), makeList("foo"));
    }

}
