/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.event;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 * Event generated when a slot changes.
 *
 * @author    Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class SlotEvent extends AbstractEvent {
    private final static int BASE = 400;
    public final static int TEMPLATE_SLOT_CLS_ADDED = BASE + 1;
    public final static int TEMPLATE_SLOT_CLS_REMOVED = BASE + 2;
    public final static int DIRECT_SUBSLOT_ADDED = BASE + 5;
    public final static int DIRECT_SUBSLOT_REMOVED = BASE + 6;
    public final static int DIRECT_SUPERSLOT_ADDED = BASE + 7;
    public final static int DIRECT_SUPERSLOT_REMOVED = BASE + 8;
    public final static int DIRECT_SUBSLOT_MOVED = BASE + 9;

    public SlotEvent(Slot slot, int type, Frame frame) {
        super(slot, type, frame);
    }

    public Cls getCls() {
        return (Cls) getArgument();
    }

    public Frame getFrame() {
        return (Frame) getArgument();
    }

    public Slot getSlot() {
        return (Slot) getSource();
    }

    public Slot getSubslot() {
        return (Slot) getArgument();
    }

    public Slot getSuperslot() {
        return (Slot) getArgument();
    }
    
    public boolean isDeletingSlotEvent() {
        return getSlot().isBeingDeleted();
    }
}
