/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.model.framestore;

import java.lang.reflect.*;
import java.util.*;

import edu.stanford.smi.protege.model.*;

public class ArgumentCheckingFrameStoreHandler extends AbstractFrameStoreInvocationHandler {

    public Object handleInvoke(Method m, Object[] args) {
        checkArguments(m, args);
        return invoke(m, args);
    }

    private void checkArguments(Method m, Object[] args) {
        if (args != null) {
            for (int i = 0; i < args.length; ++i) {
                Object arg = args[i];
                checkArgument(m, i, arg);
            }
        }
    }

    private void checkArgument(Method m, int position, Object arg) {
        if (arg == null && !allowedNull(m, position)) {
            throw new IllegalArgumentException("Null argument for method " + m.getName() + " at position: " + position);
        } else if (arg instanceof Frame) {
            checkFrame(m, position, (Frame) arg);
        } else if (arg instanceof Collection) {
            checkCollection(m, position, (Collection) arg);
        }
    }

    private void checkFrame(Method m, int position, Frame arg) {
    }

    private void checkValue(Method m, int position, Object value) {
        if (value == null) {
            throw new IllegalArgumentException("Null collection element for method " + m.getName() + " at position: " + position);
        } else if (!isValidClass(value)) {
            throw new IllegalArgumentException(
                "Invalid collection element " + value + " for method " + m.getName() + " at position: " + position);
        }
    }

    private boolean isValidClass(Object value) {
        return value instanceof Boolean
            || value instanceof Float
            || value instanceof Integer
            || value instanceof Frame
            || value instanceof String;
    }

    private void checkCollection(Method m, int position, Collection arg) {
        Iterator i = arg.iterator();
        while (i.hasNext()) {
            Object o = i.next();
            checkValue(m, position, o);
        }
    }

    private boolean allowedNull(Method m, int position) {
        String name = m.getName();
        return (name.startsWith("create") && (position == 0 || position == 1))
            || (name.startsWith("setAssociated") && position == 1)
            || (name.startsWith("setInverseSlot") && position == 1);
    }
}

