/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.model.framestore;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.logging.*;

import edu.stanford.smi.protege.util.*;

public class JournalingFrameStoreHandler extends AbstractFrameStoreInvocationHandler {
    private Logger journaler;
    private Handler handler;
    private boolean recordQueries = false;
    

    public void start(URI journalURI) {
        try {
	        handler = new FileHandler(new File(journalURI).getPath(), true);
	        journaler = Logger.getLogger("protege.journal");
	        journaler.setUseParentHandlers(false);
	        journaler.addHandler(handler);
	        journaler.setLevel(Level.ALL);
	        handler.setFormatter(new JournalFormater());
	        handler.setLevel(Level.ALL);
        } catch (IOException e) {
            Log.getLogger().throwing("JournalingFrameStoreHandler", "start", e);
        }
    }

    public void stop() {
        if (handler != null) {
            handler.flush();
            handler.close();
        }
        journaler = null;
    }

    public void handleClose() {
        stop();
    }
    
    public Object handleInvoke(Method method, Object[] args) {
        Object result = invoke(method, args);
        if (doRecord(method)) {
            record(method, args, result);
        }
        return result;
    }

    protected void record(Method m, Object[] args, Object result) {
        int length = (args == null) ? 2 : args.length + 2;
        Object[] params = new Object[length];
        params[0] = m;
        if (args != null) {
            System.arraycopy(args, 0, params, 1, args.length);
        }
        params[length-1] = result;
        journaler.log(Level.INFO, "entry", params);
    }
    protected boolean doRecord(Method method) {
        return journaler != null && (recordQueries || isModification(method));
    }
}