/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.model.framestore;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.query.*;

public interface NarrowFrameStore {

    NarrowFrameStore getDelegate();

    FrameID generateFrameID();

    int getFrameCount();
    int getClsCount();
    int getSlotCount();
    int getFacetCount();
    int getSimpleInstanceCount();
    Set getFrames();

    Frame getFrame(FrameID id);

    List getValues(Frame frame, Slot slot, Facet facet, boolean isTemplate);
    int getValuesCount(Frame frame, Slot slot, Facet facet, boolean isTemplate);
    void addValues(Frame frame, Slot slot, Facet facet, boolean isTemplate, Collection values);
    void moveValue(Frame frame, Slot slot, Facet facet, boolean isTemplate, int from, int to);
    void removeValue(Frame frame, Slot slot, Facet facet, boolean isTemplate, Object value);
    void setValues(Frame frame, Slot slot, Facet facet, boolean isTemplate, Collection values);
    Set getFrames(Slot slot, Facet facet, boolean isTemplate, Object value);
    Set getFramesWithAnyValue(Slot slot, Facet facet, boolean isTemplate);
    Set getMatchingFrames(Slot slot, Facet facet, boolean isTemplate, String value, int maxMatches);
    Set getReferences(Object value);
    Set getMatchingReferences(String value, int maxMatches);
    Set executeQuery(Query query);
    void deleteFrame(Frame frame);
    void close();

    Set getClosure(Frame frame, Slot slot, Facet facet, boolean isTemplate);

    /**
     * A complete hack to work around problems with the java packages feature
     */
    void replaceFrame(Frame frame);

    boolean beginTransaction(String name);
    boolean commitTransaction();
    boolean rollbackTransaction();
}
