/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.model.framestore;

import java.util.*;

import edu.stanford.smi.protege.model.*;

/**
 * TODO Class Comment
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
class Record {
    private Frame frame;
    private Slot slot;
    private Facet facet;
    private boolean isTemplate;
    private List values;
    
    public Record(Frame frame, Slot slot, Facet facet, boolean isTemplate, Collection values) {
        this.frame = frame;
        this.slot = slot;
        this.facet = facet;
        this.isTemplate = isTemplate;
        setValues(values);
    }
    
    public Frame getFrame() {
        return frame;
    }
    
    public Slot getSlot() {
        return slot;
    }
    
    public Facet getFacet() {
        return facet;
    }
    
    public boolean isTemplate() {
        return isTemplate;
    }
    
    public List getInternalValues() {
        return values;
    }
    
    public int getValueCount() {
        return values.size();
    }
    
    public List getValues() {
        return (values.isEmpty()) ? Collections.EMPTY_LIST : new ArrayList(values);
    }
    
    public void setValues(Collection values) {
        this.values = new ArrayList(values);
    }
    
    public void addValue(Object o) {
        values.add(o);
    }
    
    public void removeValue(Object o) {
        boolean succeeded = values.remove(o);
        if (!succeeded) {
            System.err.println("Unable to remove " + o);
        }
    }
    
    public void moveValue(int from, int to) {
        Object value = values.remove(from);
        values.add(to, value);
    }
    
    public void replaceFrame(Frame replacementFrame, boolean replaceValues) {
        if (replacementFrame.equals(frame)) {
            frame = replacementFrame;
        }
        if (replacementFrame.equals(slot)) {
            slot = (Slot) replacementFrame;
        }
        if (replacementFrame.equals(facet)) {
            facet = (Facet) replacementFrame;
        }
        if (replaceValues) {
            ListIterator i = values.listIterator();
            while (i.hasNext()) {
                Object o = i.next();
                if (replacementFrame.equals(o)) {
                    i.set(replacementFrame);
                }
            }
        }
    }
}
