/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.model.framestore;

import java.lang.reflect.*;
import java.util.*;

import edu.stanford.smi.protege.model.*;

public class TraceFrameStoreHandler extends AbstractFrameStoreInvocationHandler {
    private Map methodCounts = new HashMap();

    public Object handleInvoke(Method m, Object[] args) {
        Object o = invoke(m, args);
        trace(m, args);
        return o;
    }

    private void trace(Method method, Object[] args) {
        Integer current = (Integer) methodCounts.get(method);
        Integer next = (current == null) ? new Integer(1) : new Integer(current.intValue() + 1);
        methodCounts.put(method, next);
        print(next.intValue(), method, args);
    }
    
    public static void print(int next, Method method, Object[] args) {
        System.out.println(next + " " + method.getName() + " " + argString(args));
    }
    private static String argString(Object[] args) {
        StringBuffer buffer = new StringBuffer();
        if (args != null) {
	        for (int i = 0; i < args.length; ++i) {
	            Object o = args[i];
	            if (o instanceof Frame) {
	                o = ((Frame)o).getFrameID();
	            }
	            buffer.append(o);
	            buffer.append(" ");
	        }
        }
        return buffer.toString();
    }
}

