/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.plugin;

import java.util.*;

import edu.stanford.smi.protege.model.*;

/**
 * TODO Class Comment
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public abstract class AbstractBackendExportPlugin extends AbstractExportPlugin implements BackendExportPlugin {
    private String newProjectPath;
    
    protected AbstractBackendExportPlugin(String name) {
        super(name);
    }
    
    public boolean canExportToNewFormat(Project project) {
        return true;
    }

    public void setNewProjectPath(String newProjectPath) {
        this.newProjectPath = newProjectPath;
    }

    protected abstract void initializeSources(Project project, Collection errors);
    protected abstract void overwriteDomainInformation(Project project, Collection errors);

    public Project exportProjectToNewFormat(Project project) {
        Collection errors = new ArrayList();
        if (isCompatibleProject(project)) {
            saveAndPreserveCustomizations(project, errors);
        } else {
            saveAndDiscardCustomizations(project, errors);
        }
        Project newProject = Project.loadProjectFromFile(newProjectPath, errors);
        handleErrors(errors);
        return newProject;
    }

    protected boolean isCompatibleProject(Project project) {
        String factoryName = project.getKnowledgeBaseFactory().getClass().getName();
        return factoryName.indexOf("OWL") == -1;
    }

    private void saveAndPreserveCustomizations(Project project, Collection errors) {
        project.setProjectFilePath(newProjectPath);
        initializeSources(project, errors);
        project.save(errors);
    }

    protected void saveAndDiscardCustomizations(Project project, Collection errors) {
        Project newProject = Project.createNewProject(null, errors);
        newProject.setProjectFilePath(newProjectPath);
        newProject.save(errors);
        overwriteDomainInformation(project, errors);
    }
}
