/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.plugin;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public abstract class AbstractCreateProjectPlugin implements CreateProjectPlugin {
    private String name;
    private KnowledgeBaseFactory knowledgeBaseFactory;
    private boolean useExistingSources;

    public AbstractCreateProjectPlugin(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }
    
    public void dispose() {
        // do nothing
    }

    protected void handleErrors(Collection errors) {
        if (!errors.isEmpty()) {
            Object error = errors.iterator().next();
            Log.getLogger().severe(error.toString());
        }
    }
    
    public void setKnowledgeBaseFactory(KnowledgeBaseFactory factory) {
        this.knowledgeBaseFactory = factory;
    }
    
    public void setUseExistingSources(boolean b) {
        this.useExistingSources = b;
    }

    public Project createProject() {
        Project project = null;
        if (useExistingSources) {
            project = buildNewProject(knowledgeBaseFactory);
        } else {
            project = createNewProject(knowledgeBaseFactory);
        }
        return project;
    }
    
    protected Project createNewProject(KnowledgeBaseFactory factory) {
        Collection errors = new ArrayList();
        Project project = Project.createNewProject(factory, errors);
        handleErrors(errors);
        return project;
    }
    
    protected Project buildNewProject(KnowledgeBaseFactory factory) {
        Collection errors = new ArrayList();
        Project project = Project.createBuildProject(factory, errors);
        initializeSources(project.getSources());
        project.createDomainKnowledgeBase(factory, errors, true);
        handleErrors(errors);
        return project;
    }
    
    protected void initializeSources(PropertyList sources) {
        throw new UnsupportedOperationException();
    }
    
    
    /*
    protected void setRequiresReloadAfterCreate(boolean b) {
        requiresReloadAfterCreate = b;
    }

    public void handleImportIntoExistingProjectRequest(Project project) {
        boolean succeeded = promptForSources();
        if (succeeded) {
            Collection errors = new ArrayList();
            importIntoProject(project, errors);
            handleErrors("Failed to import project.", errors);
        }
    }
    
    public Project handleImportIntoNewProjectRequest() {
        Project project = createProject();
        handleImportIntoExistingProjectRequest(project);
        return project;
    }

    private Project createProject() {
        Collection errors = new ArrayList();
        Project project = Project.createNewProject(factory, errors);
        if (requiresReloadAfterCreate) {
            project = saveAndReloadProject(project, errors);
        }
        return project;
    }
    
    protected Project saveAndReloadProject(Project project, Collection errors) {
        PropertyList sources = project.getSources();
        KnowledgeBaseSourcesEditor editor = project.getKnowledgeBaseFactory()
                .createKnowledgeBaseSourcesEditor(null, sources);
        editor.setShowProject(true);
        JComponent parent = ProjectManager.getProjectManager().getMainPanel();
        String title = LocalizedText.getText(ResourceKey.DATABASE_CONFIGURATION_DIALOG_TITLE);
        int rval = ModalDialog.showDialog(parent, editor, title, ModalDialog.MODE_OK_CANCEL);
        if (rval == ModalDialog.OPTION_OK) {
            editor.saveContents();
            WaitCursor cursor = createWaitCursor();
            try {
                URI uri = URIUtilities.createURI(editor.getProjectPath());
                project.setProjectURI(uri);
                project.save(errors);
                String saveTitle = LocalizedText
                        .getText(ResourceKey.SAVE_PROJECT_FAILED_DIALOG_TITLE);
                handleErrors(saveTitle, errors);
                if (errors.isEmpty()) {
                    project.dispose();
                    project = Project.loadProjectFromURI(uri, errors);
                    String reloadTitle = LocalizedText
                            .getText(ResourceKey.RELOAD_PROJECT_FAILED_DIALOG_TITLE);
                    handleErrors(reloadTitle, errors);
                    if (!errors.isEmpty()) {
                        project = null;
                    }
                } else {
                    project = null;
                }
            } finally {
                cursor.hide();
            }
        }
        return project;
    }
    */

}