/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.plugin;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 * A plugin that controls the export of all or part of a knowledge to "somewhere else".
 * 
 * The export plugin ends up being a state object that is passed around among the wizard pages
 * of the configuration dialog. When the user hits "finish" the exportProject method gets called. The
 * pages are responsible for setting informatin in the exportPlugin object in their onFinish() method
 * so that the ExportPlugin.exportProject method can execute.
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public interface ExportProjectPlugin extends Plugin {
    boolean canExport(Project project);
    void exportProject(Project project);
    WizardPage createExportWizardPage(ExportWizard wizard, Project project);
}
