/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.plugin;

import javax.swing.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 * TODO Class Comment
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class ExportWizard extends Wizard {
    private Project originalProject;
    private Project newProject;
    private ExportProjectPlugin plugin;
    private boolean exportToNewFormat = false;

    public ExportWizard(JComponent owner, Project project) {
        super(owner, "Export Project");
        setSize(400, 400);
        this.originalProject = project;
        addPage(new SelectOutputFormatWizardPage(this, project));
    }

    public Project getNewProject() {
        return newProject;
    }

    public void setExportPlugin(ExportProjectPlugin plugin) {
        this.plugin = plugin;
    }

    public void setExportToNewFormat(boolean exportToNewFormat) {
        this.exportToNewFormat = exportToNewFormat;
    }

    public void onFinish() {
        super.onFinish();
        doExport();
    }

    private void doExport() {
        WaitCursor cursor = new WaitCursor(this);
        try {
            if (exportToNewFormat) {
                newProject = ((BackendExportPlugin) plugin)
                        .exportProjectToNewFormat(originalProject);
            } else {
                plugin.exportProject(originalProject);
            }
        } finally {
            cursor.hide();
        }
    }
}