/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.server;

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;

import javax.swing.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.ui.*;
import edu.stanford.smi.protege.util.*;

public class RemoteProjectManager {
    private static RemoteProjectManager _theInstance;

    public static RemoteProjectManager getInstance() {
        if (_theInstance == null) {
            try {
                RMISocketFactory.setSocketFactory(new ClientRmiSocketFactory());
            } catch (IOException e) {
                Log.exception(e, RemoteProjectManager.class, "getInstance");
            }
            _theInstance = new RemoteProjectManager();
        }
        return _theInstance;
    }

    public Project getRemoteProject() {
        JComponent mainPanel = ProjectManager.getProjectManager().getMainPanel();
        Project project = null;
        RemoteServer server = null;
        RemoteSession session = null;
        ServerPanel panel = new ServerPanel();
        String text = LocalizedText.getText(ResourceKey.REMOTE_HOST_CONNECT_DIALOG_TITLE);
        int rval = ModalDialog.showDialog(mainPanel, panel, text, ModalDialog.MODE_OK_CANCEL);
        if (rval == ModalDialog.OPTION_OK) {
            server = panel.getServer();
            session = panel.getSession();
            if (server != null && session != null) {
                project = getServerProject(mainPanel, server, session);
            }
        }
        return project;
    }

    private Project getServerProject(JComponent parent, RemoteServer server, RemoteSession session) {
        Project project = null;
        ServerProjectPanel panel = new ServerProjectPanel(server, session);
        String title = LocalizedText.getText(ResourceKey.REMOTE_PROJECT_SELECT_DIALOG_TITLE);
        int rval = ModalDialog.showDialog(parent, panel, title, ModalDialog.MODE_OK_CANCEL);
        if (rval == ModalDialog.OPTION_OK) {
            String projectName = panel.getProjectName();
            if (projectName != null) {
                project = connectToProject(server, session, projectName);
            }
        }
        return project;
    }

    public Project getProject(
        String serverName,
        String username,
        String password,
        String projectName,
        boolean pollForEvents) {
        Project p = null;
        try {
            RemoteServer server = (RemoteServer) Naming.lookup("//" + serverName + "/" + Server.getBoundName());
            if (server != null) {
                RemoteSession session = server.openSession(username, SystemUtilities.getMachineName(), password);
                if (session != null) {
                    RemoteServerProject serverProject = server.openProject(projectName, session);
                    if (serverProject != null) {
                        p = RemoteClientProject.createProject(serverProject, session, pollForEvents);
                    }
                }
            }
        } catch (Exception e) {
            Log.exception(e, this, "getProject", serverName, username, password, projectName);
        }
        return p;
    }

    private Project connectToProject(RemoteServer server, RemoteSession session, String name) {
        Project p = null;
        try {
            RemoteServerProject serverProject = server.openProject(name, session);
            p = RemoteClientProject.createProject(serverProject, session, true);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return p;
    }
}
