/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.server;

import java.net.*;
import java.rmi.*;
import java.rmi.server.*;
import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.framestore.*;

public class ServerProject extends UnicastRemoteObject implements RemoteServerProject {
    private URI _uri;
    private Server _server;
    private Project _project;
    private ServerFrameStore _domainKbFrameStore;
    private ServerFrameStore _projectKbFrameStore;

    public URI getURI(RemoteSession session) {
        return _uri;
    }

    public String getDomainKbFactoryClassName() throws RemoteException {
        return _project.getKnowledgeBase().getKnowledgeBaseFactory().getClass().getName();
    }

    public String getProjectKbFactoryClassName() throws RemoteException {
        return _project.getInternalProjectKnowledgeBase().getKnowledgeBaseFactory().getClass().getName();
    }

    public ServerProject(Server server, URI uri, Project project) throws RemoteException {
        _server = server;
        _uri = uri;
        _project = project;
        _domainKbFrameStore = createServerFrameStore(_project.getKnowledgeBase());
        _projectKbFrameStore = createServerFrameStore(_project.getInternalProjectKnowledgeBase());
    }

    private ServerFrameStore createServerFrameStore(KnowledgeBase kb) throws RemoteException {
        FrameStore fs = ((DefaultKnowledgeBase) kb).getHeadFrameStore();
        ServerFrameStore sfs = new ServerFrameStore(fs, kb);

        return sfs;
    }
    
    public RemoteServerFrameStore getDomainKbFrameStore(RemoteSession session) throws RemoteException {
        return _domainKbFrameStore;
    }

    public RemoteServerFrameStore getProjectKbFrameStore(RemoteSession session) throws RemoteException {
        return _projectKbFrameStore;
    }

    public void register(RemoteSession session) {
        _domainKbFrameStore.register(session);
        _projectKbFrameStore.register(session);
    }

    public void close(RemoteSession session) {
        _server.disconnectFromProject(this, session);
    }

    public Collection getCurrentSessions(RemoteSession unusedSession) {
        return _server.getCurrentSessions(this);
    }

    public boolean isDirty() {
        return _domainKbFrameStore.isDirty();
    }

    public void setClean() {
        _domainKbFrameStore.markClean();
        _projectKbFrameStore.markClean();
    }

}