/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.storage.database_with_include;

import java.sql.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.framestore.*;

public class DatabaseFrameStore extends SimpleFrameStore {
    private DatabaseFrameDb _databaseFrameDb;

    public DatabaseFrameStore(
        DefaultKnowledgeBase kb,
        FrameIDHelper helper,
        String driver,
        String url,
        String username,
        String password,
        String tablename,
        boolean modifiable) {
        super(
            kb,
            getBasicFrameStore(
                new DatabaseFrameDb(getFrameFactory(kb), helper, driver, url, username, password, tablename)));
        _databaseFrameDb = getDatabaseFrameDb();
        _databaseFrameDb.setModifiable(modifiable);
        // addSystemFrames();
    }

    public void close() {
        _databaseFrameDb.close();
        _databaseFrameDb = null;
    }

    private static FrameFactory getFrameFactory(DefaultKnowledgeBase kb) {
        return (kb == null) ? null : kb.getFrameFactory();
    }

    private DatabaseFrameDb getDatabaseFrameDb() {
        ClosureCachingBasicFrameStore closureDelegate = (ClosureCachingBasicFrameStore) getHelper();
        ValueCachingBasicFrameStore valuesDelegate = (ValueCachingBasicFrameStore) closureDelegate.getDelegate();
        return (DatabaseFrameDb) valuesDelegate.getDelegate();
    }

    public void saveKnowledgeBase(KnowledgeBase kb) throws SQLException {
        _databaseFrameDb.saveKnowledgeBase(kb);
    }

    public void reinitializeTable() {
        _databaseFrameDb.createNewTableAndIndices();
        setHelper(getBasicFrameStore(_databaseFrameDb));
        addSystemFrames();
    }

    private static NarrowFrameStore getBasicFrameStore(DatabaseFrameDb db) {
        return new ClosureCachingBasicFrameStore(new ValueCachingBasicFrameStore(db));
    }

    public String getTableName() {
        return getDatabaseFrameDb().getTableName();
    }

}
