/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.storage.database_with_include;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class FrameHolder {
    private Map _frameIdToFrameMap = new HashMap();
    private FrameFactory _frameFactory;

    public FrameHolder(FrameFactory factory) {
        _frameFactory = factory;
    }

    public Frame getFrame(FrameID id, Collection directTypes) {
        Frame frame = (Frame) _frameIdToFrameMap.get(id);
        if (frame == null) {
            frame = createFrame(id, directTypes);
            _frameIdToFrameMap.put(id, frame);
        }
        return frame;
    }

    private Frame createFrame(FrameID id, Collection directTypes) {
        Frame frame;
        Cls firstType = (Cls) CollectionUtilities.getFirstItem(directTypes);
        if (firstType == null || !firstType.isMetaCls()) {
            frame = _frameFactory.createSimpleInstance(id, directTypes);
        } else if (firstType.isClsMetaCls()) {
            frame = _frameFactory.createCls(id, directTypes);
        } else if (firstType.isSlotMetaCls()) {
            frame = _frameFactory.createSlot(id, directTypes);
        } else if (firstType.isFacetMetaCls()) {
            frame = _frameFactory.createFacet(id, directTypes);
        } else {
            throw new IllegalStateException("No metaclass found");
        }
        return frame;
    }
}
