/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.util;

import javax.swing.*;

/**
 * A "right mouse" popup listener for a JTree.
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public abstract class TreePopupMenuMouseListener extends PopupMenuMouseListener {

    public TreePopupMenuMouseListener(JTree tree) {
        super(tree);
    }

    /*
     * Ensure that the row at (x, y) is selected.  If the row is already selected
     * then we don't want to select it again because this would result in other 
     * selected rows being deselected.  We want to all for the possibility of
     * actions that work on multiple selections
     */
    public void setSelection(JComponent c, int x, int y) {
        JTree tree = (JTree) c;
        int row = tree.getRowForLocation(x, y);
        if (row != -1) {
	        boolean selectRow = true;
	        int[] selectedRows = tree.getSelectionRows();
	        if (selectedRows != null) {
	            for (int i = 0; i < selectedRows.length; ++i) {
	                if (row == selectedRows[i]) {
	                    selectRow = false;
	                    break;
	                }
	            }
	        }
	        if (selectRow) {
	            tree.setSelectionRow(row);
	        }
        }
    }
}