/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.util;

import java.awt.*;

import javax.swing.*;

/**
 * A Page used with the Wizard framework.
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public abstract class WizardPage extends JComponent {
    private Wizard wizard;
    private boolean pageComplete = true;
    
    public WizardPage(String name, Wizard wizard) {
        setName(name);
        this.wizard = wizard;
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    }
    
    public Wizard getWizard() {
        return wizard;
    }
    
    /**
     * Marks this page as "complete" (requiring no further input). When all pages are marked complete 
     * the Finish button is enabled. The default complete state is "true".
     */
    public void setPageComplete(boolean complete) {
        pageComplete = complete;
        wizard.notifyChanged(this);
    }
    
    public boolean isPageComplete() {
        return pageComplete;
    }
    
    /**
     * Called by the system when the Finish button is pressed. Pages should override this
     * to commit their state to the state object.
     */
    public void onFinish() {
        // do nothing
    }
    
    public void onCancel() {
        // do nothing
    }
    
    /**
     * Override to return the next page to be displayed.  Return null if no more pages are 
     * needed. The default behavior is to return null.
     */
    public WizardPage getNextPage() {
        return null;
    }
    
    /**
     * Call this when the state on a page changes in a way the next wizard page needs to be
     * changes and/or reinitialized.
     */
    public void updateNextPage() {
        wizard.updateNextPage(this);
    }
}
