/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.widget;

import java.util.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.ui.*;

/**
 * TODO Class Comment
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class DirectDomainWidget extends ClsListWidget {
    
    public void initialize() {
        super.initialize();
    }
    
    protected void handleAddAction() {
        Collection clses = getAllowedParents();
        Collection selectedClses = DisplayUtilities.pickClses(DirectDomainWidget.this, getKnowledgeBase(), clses);
        Set classSet = new HashSet(selectedClses);
        classSet.removeAll(getValues());
        addItems(classSet);
    }
    
    protected boolean canRemove(Collection values) {
        return true;
    }
    
    private Collection getAllowedParents() {
        Collection domain = new ArrayList();
        Slot slot = (Slot) getInstance();
        Iterator i = slot.getSuperslots().iterator();
        while (i.hasNext()) {
            Slot superslot = (Slot) i.next();
            Collection superslotDomain = superslot.getDirectDomain();
            domain = resolveValues(domain, superslotDomain);
        }
        if (domain.isEmpty()) {
            domain.add(getKnowledgeBase().getRootCls());
        }
        return domain;
    }
    
    private Collection resolveValues(Collection existingDomain, Collection superslotDomain) {
        return (existingDomain.isEmpty()) ? superslotDomain : existingDomain;
    }
    
    public void setValues(Collection values) {
        Slot slot = (Slot) getInstance();
        Collection domain = new ArrayList(slot.getDirectDomain());
        if (domain.isEmpty()) {
            Iterator i = slot.getSuperslots().iterator();
            while (i.hasNext()) {
                Slot superslot = (Slot) i.next();
                Collection superslotDomain = superslot.getDirectDomain();
                domain = resolveValues(domain, superslotDomain);
            }
        }
        super.setValues(domain);
    }
    
    public String getLabel() {
        return localizeStandardLabel(super.getLabel(), "Domain", ResourceKey.DOMAIN_SLOT_WIDGET_LABEL);
    }

}
