/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.htmldoc;

import java.io.*;
import java.util.*;

import edu.stanford.smi.protege.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.plugin.*;
import edu.stanford.smi.protege.util.*;

/**
 * TODO Class Comment
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class HTMLGeneratorExportPlugin extends AbstractExportPlugin {
    private Collection rootClses;
    private File outputDirectory;
    private boolean includeInstances;

    public HTMLGeneratorExportPlugin() {
        super("HTML Files");
    }

    public WizardPage createExportWizardPage(ExportWizard wizard, Project project) {
        return new GenerateHtmlOptionsWizardPage(wizard, project, this);
    }

    public static void main(String[] args) {
        Application.main(args);
    }

    public void setRootClses(Collection rootClses) {
        this.rootClses = new ArrayList(rootClses);
    }

    public void setOutputDirectory(File outputDirectory) {
        this.outputDirectory = outputDirectory;
    }

    public void setIncludeInstances(boolean includeInstances) {
        this.includeInstances = includeInstances;
    }

    public void exportProject(Project project) {
        outputDirectory.mkdirs();
        ProtegeGenClassHierarchy.generateDocs(project.getKnowledgeBase(), rootClses, true,
                "index.html", outputDirectory.getPath(), includeInstances);
    }

}